@@ -625,11 +625,17 @@ QAbstractAxisPrivate::~QAbstractAxisPrivate()
         m_graph->removeAxis(q);
 }
 
+void QAbstractAxisPrivate::setGraph(QGraphsView *graph)
+{
+    if (m_graph && graph)
+        qCWarning(lcAxis2D, "%p axis already associated with %p", this, m_graph);
+
+    m_graph = graph;
+}
+
 void QAbstractAxisPrivate::handleRangeChanged(qreal min, qreal max)
 {
     setRange(min,max);
 }
 
 QT_END_NAMESPACE
-
-
