@@ -11,9 +11,28 @@
 #include <private/qgraphsview_p.h>
 #include <private/qxyseries_p.h>
 #include <QtQuick/private/qquicktaphandler_p.h>
+#include <qtgraphs_tracepoints_p.h>
 
 QT_BEGIN_NAMESPACE
 
+Q_TRACE_PREFIX(qtgraphs,
+              "QT_BEGIN_NAMESPACE" \
+              "class AreaRenderer;" \
+              "QT_END_NAMESPACE"
+          )
+
+Q_TRACE_POINT(qtgraphs, QGraphs2DAreaRenderePointInArea_entry, int x, int y);
+Q_TRACE_POINT(qtgraphs, QGraphs2DAreaRenderePointInArea_exit);
+
+Q_TRACE_POINT(qtgraphs, QGraphs2DAreaRendereAfterPolish_entry);
+Q_TRACE_POINT(qtgraphs, QGraphs2DAreaRendereAfterPolish_exit);
+
+Q_TRACE_POINT(qtgraphs, QGraphs2DAreaRendererCalculateSeriesU_entry);
+Q_TRACE_POINT(qtgraphs, QGraphs2DAreaRendererCalculateSeriesU_exit);
+
+Q_TRACE_POINT(qtgraphs, QGraphs2DAreaRendererCalculateSeriesL_entry);
+Q_TRACE_POINT(qtgraphs, QGraphs2DAreaRendererCalculateSeriesL_exit);
+
 AreaRenderer::AreaRenderer(QGraphsView *graph, bool clipPlotArea)
     : QQuickItem(graph)
     , m_graph(graph)
@@ -169,6 +188,7 @@ void AreaRenderer::handlePolish(QAreaSeries *series)
 
     if (series->isVisible()) {
         qreal prevUpperY = 0;
+        Q_TRACE_SCOPE(QGraphs2DAreaRendererCalculateSeriesU);
         for (int i = 0, j = 0; i < upperPoints.size() + extraPointCount; ++i, ++j) {
             qreal x;
             qreal y;
@@ -221,6 +241,7 @@ void AreaRenderer::handlePolish(QAreaSeries *series)
     }
 
     if (lower && series->isVisible()) {
+        Q_TRACE_SCOPE(QGraphs2DAreaRendererCalculateSeriesL);
         auto &&lowerPoints = lower->points();
         QList<QPointF> fittedPoints;
 #ifdef USE_SPLINEGRAPH
@@ -269,6 +290,8 @@ void AreaRenderer::handlePolish(QAreaSeries *series)
 
 void AreaRenderer::afterPolish(QList<QAbstractSeries *> &cleanupSeries)
 {
+    Q_TRACE_SCOPE(QGraphs2DAreaRendereAfterPolish);
+
     for (auto series : cleanupSeries) {
         auto areaSeries = qobject_cast<QAreaSeries *>(series);
         if (areaSeries && m_groups.contains(areaSeries)) {
@@ -318,6 +341,7 @@ bool pointInTriangle(QPoint pt, QPoint v1, QPoint v2, QPoint v3)
 
 bool AreaRenderer::pointInArea(QPoint pt, QAreaSeries *series) const
 {
+    Q_TRACE_SCOPE(QGraphs2DAreaRenderePointInArea, pt.x(), pt.y());
     QList<QPointF> upperPoints = series->upperSeries()->points();
     QList<QPointF> lowerPoints;
 
